using System;
using System.Data;
using System.Data.SqlClient;
using System.Configuration;
using System.Xml;

namespace AcmeDirectory
{
   /// <summary>
   /// Summary description for DataAccess.
   /// </summary>
   public class DataManager
   {
      private SqlConnection   dataCN;
      private SqlTransaction  dataTX;

      public DataManager()
      {
         // initialize the connection object
         dataCN = new SqlConnection();
         dataCN.ConnectionString = ConfigurationSettings.AppSettings.Get("connString");
      }

      //******************************************************************************
      // Method:       Open
      //
      // Description:  Open a database connection... if beginTrans is true than
      //               a new transaction will be started.
      //
      //******************************************************************************
      public void Open()
      {
         Open( false );
      }
      public void Open( bool beginTrans )
      {
         if( dataCN.State == System.Data.ConnectionState.Closed)
         {
            dataCN.Open();
         }
         if( beginTrans == true)
         {
            dataTX = dataCN.BeginTransaction();
         }
      }


      //******************************************************************************
      // Method:      Close
      //
      // Description: Close the database connection. If commit is true than any
      //              open transaction will also be comitted.
      //
      //******************************************************************************
      public void Close()
      {
         Close( true );
      }
      public void Close( bool commitTrans )
      {
         // make sure the connection is open
         if( dataCN.State == System.Data.ConnectionState.Open )
         {
            // check to see if we have a transaction going
            if( dataTX != null )
            {
               if( commitTrans == true )
               {
                  dataTX.Commit();
               }
               else
               {
                  dataTX.Rollback();
               }
            }
         }
         // close the connection
         dataCN.Close();
      }
      //******************************************************************************
      // Method:      NewQueryDA
      //
      // Description: Used to initialize a Query based DataAdaptor.
      //
      //******************************************************************************
      public SqlDataAdapter NewQueryDA( string query )
      {
         // Initalize a DataAdapter and set the query
         SqlDataAdapter queryDA = new SqlDataAdapter(query, dataCN);
         return queryDA;
      }

      //******************************************************************************
      // Method:      NewProcDA
      //
      // Description: Used to initialize a Stored Proc based DataAdaptor.
      //
      //******************************************************************************
      public SqlDataAdapter NewProcDA( string procedure, params object[] procParams )
      {
         // Initalize a DataAdapter and set it as a stored procedure
         SqlDataAdapter procDA = new SqlDataAdapter();
         procDA.SelectCommand = new SqlCommand(procedure, dataCN);
         procDA.SelectCommand.CommandType = CommandType.StoredProcedure;

         // now add any parameters that were passed in
         for( int index = 0; index < procParams.Length; index+=2 )
         {
            SqlParameter sqlParam = new SqlParameter();
            sqlParam.ParameterName = procParams[index].ToString();
            sqlParam.Value = procParams[index+1];
            sqlParam.SqlDbType = GetSQLType(sqlParam.Value);
            procDA.SelectCommand.Parameters.Add( sqlParam );
         }
         // return the DataAdapter
         return procDA;
      }
      //******************************************************************************
      // Method:      ExecuteCommand
      //
      // Description: Used to Execute a command with a DataAdaptor that
      //              was initialized with one of the Init functions.
      //
      //******************************************************************************
      public int ExecuteCommand( SqlDataAdapter cmdDA )
      {
         SqlParameter sqlParam = new SqlParameter();
         sqlParam.SqlDbType = System.Data.SqlDbType.Int;
         sqlParam.Value = 0;
         sqlParam.Direction = System.Data.ParameterDirection.ReturnValue;
         cmdDA.SelectCommand.Parameters.Add( sqlParam );

         // execute the command
         cmdDA.SelectCommand.ExecuteNonQuery();
         return int.Parse( sqlParam.Value.ToString());
      }

      //******************************************************************************
      // Method:      GetSQLType
      //
      // Description: Helper used to get the correct SQL data type for the 
      //              value type passed in
      //
      //******************************************************************************
      private SqlDbType GetSQLType(object procValue )
      {
         // Handle different types... default to varchar
         SqlDbType result;
         TypeCode paramCode = Type.GetTypeCode(procValue.GetType());
         switch(paramCode)
         {
            case TypeCode.Boolean:
               result = SqlDbType.Bit;
               break;
            case TypeCode.Byte:
               result = SqlDbType.TinyInt;
               break;
            case TypeCode.Char:
               result = SqlDbType.Char;
               break;
            case TypeCode.DateTime:
               result = SqlDbType.DateTime;
               break;
            case TypeCode.Decimal:
               result = SqlDbType.Decimal;
               break;
            case TypeCode.Double:
               result = SqlDbType.Float;
               break;
            case TypeCode.Int16:
               result = SqlDbType.Int;
               break;
            case TypeCode.Int32:
               result = SqlDbType.Int;
               break;
            case TypeCode.Int64:
               result = SqlDbType.Int;
               break;
            case TypeCode.Object:
               result = SqlDbType.Variant;
               break;
            case TypeCode.SByte:
               result = SqlDbType.TinyInt;
               break;
            case TypeCode.Single:
               result = SqlDbType.Real;
               break;
            default:
               result = SqlDbType.VarChar;
               break;
         }
         // return the type
         return result;
      }

      //******************************************************************************
      // Method:      GetXML
      //
      // Description: Uses a TextReader to get the XML from the DataSet with
      //              all of the namespace info stripped off.
      //
      //******************************************************************************
      public string GetXML( DataSet inputDS )
      {
         return GetXML(inputDS, null );
      }
      public string GetXML( DataSet inputDS, string xpath )
      {
         // load the data into a text reader
         System.IO.MemoryStream buff = new System.IO.MemoryStream();
         inputDS.WriteXml( buff, System.Data.XmlWriteMode.IgnoreSchema );
         buff.Seek(0, System.IO.SeekOrigin.Begin );
         XmlReader reader = new XmlTextReader(buff);

         // initialize a buffer
         System.Text.StringBuilder xmlBuff = new System.Text.StringBuilder();
         
         // Read each node in the tree.
         while( reader.Read())
         {
            switch( reader.NodeType )
            {
               case XmlNodeType.Element:
                  xmlBuff.Append("<" + reader.Name);
                  bool isEmpty = reader.IsEmptyElement;
                  while( reader.MoveToNextAttribute())
                  {
                     // don't add any namespace attributes
                     if( !reader.Name.Equals("xmlns"))
                     {
                        xmlBuff.Append(" " + reader.Name + "='" + reader.Value + "'");
                     }
                  }
                  if( isEmpty == true )
                     xmlBuff.Append(" />");
                  else
                     xmlBuff.Append(">");
                  break;
               case XmlNodeType.Text:
                  xmlBuff.Append(reader.Value);
                  break;
               case XmlNodeType.CDATA:
                  xmlBuff.Append(reader.Value);
                  break;
               case XmlNodeType.ProcessingInstruction:
                  xmlBuff.Append("<?" + reader.Name + " " + reader.Value + "?>");
                  break;
               case XmlNodeType.Comment:
                  //xmlBuff.Append("<!--" + reader.Value + "-->");
                  break;
               case XmlNodeType.Document:
                  xmlBuff.Append("<?xml version='1.0'?>");
                  break;
               case XmlNodeType.Whitespace:
                  xmlBuff.Append(reader.Value);
                  break;
               case XmlNodeType.SignificantWhitespace:
                  xmlBuff.Append(reader.Value);
                  break;
               case XmlNodeType.EndElement:
                  xmlBuff.Append("</" + reader.Name + ">");
                  break;
            }
         }
         // check to see if a specific node was requested
         if( xpath != null )
         {
            XmlDocument xmlDoc = new XmlDocument();
            xmlDoc.LoadXml( xmlBuff.ToString());
            XmlNode node = xmlDoc.SelectSingleNode(xpath);
            if( node != null )
            {
               return node.OuterXml;
            }
         }
         // just return the string
         return xmlBuff.ToString();
      }
   }
}
