using System.Data.SqlClient;
using System.Configuration;

Namespace Common
   Public Class FMSDataAccess
      Private dataCN As SqlConnection
      Private dataTX As SqlTransaction

      '******************************************************************************
      ' Method:      New
      '
      ' Description: create the connection during initialization 
      '
      '******************************************************************************
      Public Sub New()
         dataCN = New SqlConnection()
         dataCN.ConnectionString = ConfigurationSettings.AppSettings("connString")
      End Sub

      '******************************************************************************
      ' Method:       Open
      '
      ' Description:  Open a database connection... if beginTrans is true than
      '               a new transaction will be started.
      '
      '******************************************************************************
      Public Sub Open(Optional ByVal beginTrans As Boolean = False)
         If dataCN.State = ConnectionState.Closed Then
            dataCN.Open()
            If beginTrans = True Then
               dataTX = dataCN.BeginTransaction()
            End If
         End If
      End Sub

      '******************************************************************************
      ' Method:      Close
      '
      ' Description: Close the database connection. If commit is true than any
      '              open transaction will also be comitted.
      '
      '******************************************************************************
      Public Sub Close(Optional ByVal commitTrans As Boolean = True)
         ' make sure the connection is open
         If dataCN.State = ConnectionState.Open Then
            ' check to see if we have a transaction going
            If Not dataTX Is Nothing Then
               If commitTrans = True Then
                  dataTX.Commit()
               Else
                  dataTX.Rollback()
               End If
            End If
            ' close the connection
            dataCN.Close()
         End If
      End Sub

      '******************************************************************************
      ' Method:      InitializeQuery
      '
      ' Description: Initialize a Query and return the DataAdapter 
      '
      '******************************************************************************
      Public Function InitializeQuery(ByVal query As String) As SqlDataAdapter
         ' Initalize a DataAdapter and set the query
         Dim data As SqlDataAdapter = New SqlDataAdapter(query, dataCN)
         Return data
      End Function

      '******************************************************************************
      ' Method:      InitializeProc
      '
      ' Description: Initialize a stored procedure and return the DataAdapter 
      '
      '******************************************************************************
      Public Function InitializeProc(ByVal procedure As String, ByVal ParamArray params() As Object) As SqlDataAdapter
         ' Initalize a DataAdapter and set the query
         Dim data As SqlDataAdapter = New SqlDataAdapter()
         data.SelectCommand = New SqlCommand(procedure, dataCN)
         data.SelectCommand.CommandType = CommandType.StoredProcedure

         ' loop and add the parameters
         Dim index, count As Integer
         count = params.Length
         If count > 0 Then
            For index = 0 To count - 1 Step 2
               ' create a parameter and initialize it
               Dim param As New SqlParameter()
               param.ParameterName = CType(params.GetValue(index), String)
               param.Value = params.GetValue(index + 1)
               param.SqlDbType = GetSQLType(param.Value)

               ' add the parameter
               data.SelectCommand.Parameters.Add(param)
            Next
         End If
         ' return the DataAdapter
         Return data
      End Function

      '******************************************************************************
      ' Method:      ExecuteCommand
      '
      ' Description: Used to execute a command with a data adaptor that has been
      '              initialized with one of the Initialize routines
      '
      '******************************************************************************
      Public Function ExecuteCommand(ByVal commandDA As SqlDataAdapter) As Integer
         ' default
         Dim returnVal As Integer = 0

         ' Execute the query and see if there was a return value
         commandDA.SelectCommand.ExecuteNonQuery()
         Dim param As SqlParameter
         For Each param In commandDA.SelectCommand.Parameters
            If param.Direction = ParameterDirection.ReturnValue Then
               returnVal = CType(param.Value, Integer)
               Exit For
            End If
         Next
         ' return 
         Return returnVal
      End Function

      '******************************************************************************
      ' Method:      GetSQLType
      '
      ' Description: Helper used to get the correct SQL data type for the 
      '              value type passed in
      '
      '******************************************************************************
      Private Function GetSQLType(ByVal value As Object) As SqlDbType
         ' Handle different types... default to varchar
         Dim result As SqlDbType
         Dim paramType As Type = value.GetType()
         Select Case Type.GetTypeCode(paramType)
            Case TypeCode.Boolean
               result = SqlDbType.Bit
            Case TypeCode.Byte
               result = SqlDbType.TinyInt
            Case TypeCode.Char
               result = SqlDbType.Char
            Case TypeCode.DateTime
               result = SqlDbType.DateTime
            Case TypeCode.Decimal
               result = SqlDbType.Decimal
            Case TypeCode.Double
               result = SqlDbType.Float
            Case TypeCode.Int16
               result = SqlDbType.Int
            Case TypeCode.Int32
               result = SqlDbType.Int
            Case TypeCode.Int64
               result = SqlDbType.Int
            Case TypeCode.Object
               result = SqlDbType.Variant
            Case TypeCode.SByte
               result = SqlDbType.TinyInt
            Case TypeCode.Single
               result = SqlDbType.Real
            Case Else
               result = SqlDbType.VarChar
         End Select
         ' return the type
         Return result
      End Function
   End Class
End Namespace
